import os
from pathlib import Path

from flask import Flask, Response, request, jsonify, send_file, render_template
app = Flask(__name__)

from wizard import tts
from wizard import utils
from wizard.playlist import create_playlist

def streamwav(name):
    def generate():
        with open(f"wizard/playlist/{name}.wav", "rb") as fwav:
            data = fwav.read(1024)
            while data:
                yield data
                data = fwav.read(1024)
    return Response(generate(), mimetype="audio/x-wav")

@app.route("/add-file", methods=["POST"])
def add():
    lines = request.json.get("lines")
    file_name = request.json.get("file_name")
    if lines:
        Path(f'wizard/static/audio/{file_name}').mkdir(parents=True, exist_ok=True)
        utils.clear_directory(f'wizard/static/audio/{file_name}')
        for idx, line in enumerate(lines):
            speech = tts.text_to_speech(line)
            tts.speech_to_file(speech, f"wizard/static/audio/{file_name}/{file_name}-{str(idx+1)}.mp3")
        return jsonify({"message" : "OK"})
    else:
        return jsonify({"message" : "FAILURE"})

@app.route("/play/<name>")
def play(name):
    names = [i.replace(".mp3", "") for i in utils.list_files(f"wizard/static/audio/{name}")]
    names = sorted(names, key=lambda x : int(x.split("-")[1]))
    print(names)
    return render_template("player.html", names=names)

@app.route("/add-playlist", methods=["POST"])
def add_playlist():
    data = request.json
    d = utils.json_to_dict(utils.DATA_PATH)
    if not "playlists" in d:
        d["playlists"] = {}
    d["playlists"][data.get("name")] = data.get("files")
    utils.dict_to_json(d, utils.DATA_PATH)
    return jsonify({"message" : "OK"})

@app.route("/files", methods=["GET"])
def files():
    f = utils.list_files("wizard/static/audio")
    return jsonify({"files" : f})




import os
import json
from pathlib import Path
from pydub import AudioSegment
import re
import shutil


HOME = os.path.expanduser('~')
DATA_PATH = f"{HOME}/.wizard/data.json"

def clear_directory(path):
    shutil.rmtree(path)
    Path(path).mkdir(parents=True, exist_ok=True)

def split_sentences(st): 
    sentences = re.split(r'[.?!]\s*', st)
    if sentences[-1]:
        return sentences
    else:
        return sentences[:-1]

def clean_string(string):
    return string.replace("\n", " ")

def file_to_text(path):
    with open(path) as f:
        text = f.read()
    return text

def get_file_name(path):
    p = Path(path)
    return p.stem

def dict_to_json(d, path):
    with open(path, "w") as f: 
        json.dump(d, f)

def json_to_dict(path):
    with open(path) as f:
        d = json.load(f)
    return d

def list_files(path):
    return os.listdir(path)

def get_url():
    return json_to_dict(DATA_PATH).get("server")

def mp3_to_wav(mp3_path, wav_path):
    sound = AudioSegment.from_mp3(mp3_path)
    sound.export(wav_path, format="wav")


import click
import requests as r
from pathlib import Path

from wizard.server import app
from wizard import utils
from wizard import term

@click.group()
def cli():
    pass

@click.command()
@click.option("--host", default="127.0.0.1", help="Server host")
@click.option("--port", default="8040", help="Server port")
def server(host, port):
    Path('wizard/audio').mkdir(parents=True, exist_ok=True)
    app.run(host, port, debug=True)

@click.command()
def init():
    Path(f'{utils.HOME}/.wizard').mkdir(parents=True, exist_ok=True)
    ip = input("Type in host and port of server: ")
    try:
        d = utils.json_to_dict(utils.DATA_PATH)
    except:
        d = {}
    d["server"] = ip
    utils.dict_to_json(d, utils.DATA_PATH)
    click.echo(f'Saved configuration.')

@click.command()
@click.argument('file')
@click.option('--d', default=".")
def add(file, d):
    text = utils.file_to_text(file)
    text = utils.clean_string(text)
    lines = [i for i in utils.split_sentences(text) if i.strip()] 
    file_name = utils.get_file_name(file)
    url = utils.get_url()
    if url:
        res = r.post(f'http://{url}/add-file', json={"lines":lines, "file_name":file_name})
        if res.json().get("message") == "OK":
            click.echo(f'Added {file} to audio list.')
        else:
            click.echo("Error occurred.")
    else:
        click.echo("Error occurred.")

@click.command()
@click.argument('name')
def playlist(name):
    url = utils.get_url()
    res = r.get(f'http://{url}/files')
    files = res.json().get("files")
    if files:
        selected = term.multi_select(files, "Select audio files to add to playlist: ")
        if selected:
            r.post(f"http://{url}/add-playlist", json={"name" : name, "files" : selected})
            click.echo(f"Added new playlist: {name}")
        else:
            click.echo("Error occurred.")
    else:
        click.echo("Error occurred.")

cli.add_command(init)
cli.add_command(playlist)
cli.add_command(server)
cli.add_command(add)



from simple_term_menu import TerminalMenu

def multi_select(options, title):
    terminal_menu = TerminalMenu(
        options,
        title=title,
        multi_select=True,
        show_multi_select_hint=True,
        show_search_hint=True
    )
    menu_entry_indices = terminal_menu.show()
    return terminal_menu.chosen_menu_entries



from wizard import utils
from wizard import tts
from pydub import AudioSegment
from pathlib import Path

def create_playlist(name, length=5):
    data = utils.json_to_dict(utils.DATA_PATH)
    playlists = data.get("playlists")
    if playlists:
        output = AudioSegment.silent(duration=1000)
        for file in playlists.get(name, []):
            no_ex = file.replace(".wav", "")
            speech = tts.text_to_speech(no_ex)
            tts.speech_to_file(speech, f"wizard/tmp/{no_ex}.mp3")
            utils.mp3_to_wav(f"wizard/tmp/{no_ex}.mp3", f"wizard/tmp/{no_ex}.wav")
            title = AudioSegment.from_wav(f"wizard/tmp/{no_ex}.wav")
            wav = AudioSegment.from_wav(f"wizard/audio/{file}")
            output += title
            output += AudioSegment.silent(duration=2000)
            output += wav 
            output += AudioSegment.silent(duration=2000)
    repeat = int(float(length * 60) / float(len(output) / 1000))
    output = output * max([repeat,1])
    Path('wizard/playlist').mkdir(parents=True, exist_ok=True)
    output.export(f"wizard/playlist/{name}.wav", format="wav")


from setuptools import setup, find_packages

setup(
    name='memorize-wizard',
    version='0.1.1',
    packages=find_packages(), 
    include_package_data=True, 
    install_requires=[
        "Flask==1.1.2", 
        "click==8.0.1", 
        "gTTS==2.2.2", 
        "requests==2.25.1", 
        "simple-term-menu==1.2.1", 
        "pydub==0.25.1"
    ],
    entry_points={
        'console_scripts': [
            'wizard = wizard.main:cli',
        ],
    },
)
